/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */



var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

var type = {};
var types = 'Array Object String Date RegExp Function Boolean Number Error Null Undefined'.split(' ');
/**
 * Check the type of a value.
 */
type.is = function (type, val) {
    return Object.prototype.toString.call(val).slice(8, -1) === type;
};
// Populate helper methods.
types.forEach(function (t) { return (type["is" + t] = function (v) { return type.is(t, v); }); });

/**
 * Return the value or execute as a function
 */
function value(val) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    return type.isFunction(val) ? val.apply(this, args) : val;
}

/**
 * Serialize and Unserialize data
 */
var JSONSerializer = {
    /**
     * Serialize the provided data.
     */
    serialize: function (data) {
        try {
            return JSON.stringify(data);
        }
        catch (e) {
            return data;
        }
    },
    /**
     * Unserialize the provided data.
     */
    unserialize: function (payload) {
        try {
            return JSON.parse(payload);
        }
        catch (e) {
            return payload;
        }
    }
};

/**
 * A decorator for a Storage implementation
 */
var Store = /** @class */ (function () {
    /**
     * Create a Store instance.
     */
    function Store(driver, serializer) {
        this.driver = driver;
        this.serializer = serializer || JSONSerializer;
    }
    /**
     * Add a key/value pair to storage
     */
    Store.prototype.setItem = function (key, val) {
        try {
            this.driver.setItem(key, this.serializer.serialize(val));
        }
        catch (e) {
            if ([
                'QUOTA_EXCEEDED_ERR',
                'NS_ERROR_DOM_QUOTA_REACHED',
                'QuotaExceededError'
            ].indexOf(e.name) !== -1) {
                throw new Error('[lkr] The Storage quota has been exceeded');
            }
            else {
                throw new Error("[lkr] Could not add item with key \"" + key + "\"");
            }
        }
        return this;
    };
    /**
     * Retrieve a value from storage by key
     */
    Store.prototype.getItem = function (key) {
        return this.serializer.unserialize(this.driver.getItem(key));
    };
    /**
     * Determine whether the key exists in storage.
     */
    Store.prototype.hasItem = function (key) {
        return this.driver.hasOwnProperty(key);
    };
    /**
     * Remove an item from storage.
     */
    Store.prototype.removeItem = function (key) {
        if (!this.hasItem(key))
            return false;
        this.driver.removeItem(key);
        return true;
    };
    /**
     * Remove all items from storage.
     */
    Store.prototype.clear = function () {
        this.driver.clear();
        return this;
    };
    /**
     * Iterate through the items in storage and execute the callback.
     */
    Store.prototype.forEach = function (callback, thisContext) {
        if (thisContext === void 0) { thisContext = this; }
        for (var item in this.driver) {
            callback.call(thisContext, this.driver[item], item);
        }
    };
    /**
     * Determine whether the storage driver is supported.
     */
    Store.prototype.isSupported = function () {
        var l = 'l';
        try {
            this.driver.setItem(l, l);
            this.driver.removeItem(l);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    Object.defineProperty(Store.prototype, "length", {
        /**
         * Get the number of items in storage.
         */
        get: function () {
            return this.driver.length;
        },
        enumerable: true,
        configurable: true
    });
    return Store;
}());

/**
 * A Fluent Storage API.
 */
var Lkr = /** @class */ (function () {
    /**
     * Create a Lkr instance.
     */
    function Lkr(options) {
        /**
         * The configuration options.
         */
        this.opts = options;
        /**
         * The Store instance.
         */
        this.store = (function (driver, self) {
            if (!options.drivers.hasOwnProperty(driver)) {
                throw new Error("[lkr] Driver \"" + driver + "\" not available.");
            }
            var store = new Store(options.drivers[driver], options.serializer);
            if (!store.isSupported()) {
                throw new Error("[lkr] Driver \"" + driver + "\" not supported.");
            }
            return store;
        })(options.driver, this);
    }
    Object.defineProperty(Lkr.prototype, "options", {
        /**
         * Get the options.
         */
        get: function () {
            return this.opts;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Lkr.prototype, "prefix", {
        /**
         * Get the prefix.
         */
        get: function () {
            return "" + this.opts.namespace + this.opts.separator;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Get the fully qualified key.
     */
    Lkr.prototype.getKey = function (key) {
        return this.opts.namespace ? "" + this.prefix + key : key;
    };
    /**
     * Add a new item to storage even if it already exists.
     */
    Lkr.prototype.put = function (key, val, def) {
        if (type.isUndefined(key))
            throw new Error('[lkr] You must specify a key.');
        key = value(key);
        if (type.isObject(key)) {
            for (var item in key) {
                var v = value(key[item]);
                this.store.setItem(this.getKey(item), type.isUndefined(v) ? def : v);
            }
        }
        else {
            if (type.isUndefined(val))
                throw new Error('[lkr] You must specify a value.');
            this.store.setItem(this.getKey(key), value(val, this.get(key, def)));
        }
        return this;
    };
    /**
     * Add an item to storage if it doesn't already exist.
     */
    Lkr.prototype.add = function (key, value$$1, def) {
        if (!this.has(key)) {
            this.put(key, value$$1, def);
            return true;
        }
        return false;
    };
    /**
     * Retrieve the specified item from storage.
     */
    Lkr.prototype.get = function (key, def) {
        key = value(key);
        if (type.isArray(key)) {
            var items = {};
            for (var _i = 0, key_1 = key; _i < key_1.length; _i++) {
                var k = key_1[_i];
                items[k] = this.get(k, def);
            }
            return items;
        }
        if (!this.has(key))
            return arguments.length === 2 ? def : void 0;
        return this.store.getItem(this.getKey(key));
    };
    /**
     * Determine whether the item exists in storage.
     */
    Lkr.prototype.has = function (key) {
        return this.store.hasItem(value(this.getKey(key)));
    };
    /**
     * Remove specified item(s) from storage.
     */
    Lkr.prototype.forget = function (key) {
        key = value(key);
        if (type.isArray(key)) {
            key.map(this.forget, this);
        }
        else {
            this.store.removeItem(this.getKey(key));
        }
        return this;
    };
    /**
     * Retrieve the specified item from storage and then remove it
     */
    Lkr.prototype.pull = function (key, def) {
        var value$$1 = this.get(key, def);
        this.forget(key);
        return value$$1;
    };
    /**
     * Get all the items in storage within the currently set namespace/driver.
     */
    Lkr.prototype.all = function () {
        var _this = this;
        var items = {};
        this.store.forEach(function (val, key) {
            if (_this.opts.namespace) {
                if (key.indexOf(_this.prefix) === 0)
                    key = key.substring(_this.prefix.length);
            }
            if (_this.has(key))
                items[key] = _this.get(key);
        }, this);
        return items;
    };
    /**
     * Iterate through the items within the current namespace/driver
     * and execute the callback.
     */
    Lkr.prototype.each = function (callback, thisContext) {
        if (thisContext === void 0) { thisContext = this; }
        var items = this.all();
        for (var item in items) {
            callback.call(thisContext, items[item], item);
        }
    };
    /**
     * Get the storage keys as an array.
     */
    Lkr.prototype.keys = function () {
        return Object.keys(this.all());
    };
    /**
     * Remove all items set within the current namespace/driver.
     */
    Lkr.prototype.clean = function () {
        return this.forget(this.keys());
    };
    /**
     * Empty the current storage driver completely. Careful now.
     */
    Lkr.prototype.empty = function () {
        this.store.clear();
        return this;
    };
    /**
     * Get the total number of items within the current namespace/driver.
     */
    Lkr.prototype.count = function () {
        return this.keys().length;
    };
    /**
     * Set the driver by key.
     */
    Lkr.prototype.driver = function (driver) {
        return Lkr.make(__assign({}, this.opts, { driver: driver }));
    };
    /**
     * Set the namespace.
     */
    Lkr.prototype.namespace = function (namespace) {
        return Lkr.make(__assign({}, this.opts, { namespace: namespace }));
    };
    /**
     * Create a new instance of Lkr.
     */
    Lkr.make = function (options) {
        return new Lkr(options);
    };
    return Lkr;
}());

var Locker = new Lkr({
    drivers: {
        local: window.localStorage,
        session: window.sessionStorage
    },
    driver: 'local',
    namespace: 'lkr',
    separator: '.'
});

export { Lkr, Locker };
//# sourceMappingURL=lkr.es.js.map
